<?php
/*
    ----------------------------- LICENCE -----------------------------

    Copyright 2008  Fabio Bernasconi  (email : fabio@fabiobernasconi.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

define("OPTION_ROOT_DIR", "ad_rootDir");

/**
 * Parses a function definition string and calls it. By definition the 
 * function call must be in the following format 
 * ad__functionName(relativePath, sort). A call to ad_parseFunDefinition
 * will, by default, return an associative array whith indices 'functionName', 
 * which holds the name of the callable function and 'functionArgs'. The value
 * of 'functionArgs' is again an array: array(0) is the relative path, array(1)
 * is the name of the function used by usort. If the second argument is missing
 * the default sortfunction is applied ad_sortFilesBy_date).
 *
 * @param $functionDefinition String denoting a function: ad_fn(param1, param2)
 * @return An associative array whith indices 'functionName' and 'functionArgs'
 *         or false if the function prefix is not ad__
 */
function ad_parseFunDefinition($functionDefinition)
{
	// simple security check. Functions starting with ad__ are callable only!
	if(strpos($functionDefinition, "ad__") != 0)
	{
		return false;
	}

	// find position of parenthesis
	$parIndex = strpos(trim($functionDefinition), "("); 

	// get the function name
	$fn = substr($functionDefinition, 0, $parIndex);


	// get the argument string
	$argsString = substr($functionDefinition, $parIndex + 1, -1);
	// split into separate arguments and trim each of them...
	$args = split(",", $argsString);
	array_walk($args, "ad_myTrim");

	if(empty($args[1]))
	{
		$args[1] = "ad_sortFilesBy_date";
	} else
	{
		$args[1] = "ad_sortFilesBy_" . $args[1];
	}

	return array('functionName' => $fn, 'functionArgs' => $args);
}

/**
 * List the files in a directory (files only). The path is a combination of
 * the path set in the Autodownload defaults and the $dir argument.
 *
 * @param $dir The directory to list from (relative to the download path 
 *             set in plugin options page)
 * @return A list of files where each entry is an associative array 
 *             with 'path' and 'file' keys
 */
function ad_listFiles($dir)
{
	$url = ad_getDownloadURL($dir);
	$absDir = ad_getDownloadDir($dir) . "/";

	if(!ad_isDirectory($absDir))
	{
		return array();
	}

	$handle = opendir($absDir);

	// get each entry
	$files = array();
	while (false !== ($file = readdir($handle))) 
	{
		if($file == "." || $file == "..")
		{
			continue;
		}

		$absFile = $absDir . $file;
	
		if(is_file($absFile))
		{
			$tmp['path'] = ad_getDownloadUrl($dir . "/" . $file);
			$tmp['name'] = $file;
			$tmp['date'] = filemtime($absFile);
			$files[] = $tmp;
		}
	}

	// close directory
	closedir($handle);
	
	return $files;
}

/**
 * Links all occurrences of file names in the post/page content.
 * 
 * @param $content The document content
 * @param $files An array of file names to be replaced by links
 * @return The content with all occurrences of file names replaced
 */
function ad_linkFilesInContent($content, $files)
{
	// FIXME if linkFilesInContent is called more than once with the same 
	// arguments it will replace the same string over and over!!! must be fixed
	foreach($files as $file)
	{
		$linked = sprintf('<a href="%s" target="_blank">%s</a>', $file['path'], $file['name']);
		$content = ereg_replace($file['name'], $linked, $content);
	}
	
	return $content;
}


/* HELPER FUNCTIONS */

/**
 * Get the absolute path to a directory.
 *
 * @param $dir A relative path that will be combined with the root path defined
 *             in the autodownload settings
 * @return The newly generated absolute path
 */
function ad_getDownloadDir($dir)
{
	$absPath = ABSPATH . get_option(OPTION_ROOT_DIR);
	return $absPath . "/" . ad_stripTrailingSlash($dir);
}

/**
 * Checks whether a path is a directory or not.
 *
 * @param $path The path that should denote a directory
 * @return True if $path points to a valid directory, false otherwise
 */
function ad_isDirectory($path)
{
	return is_dir($path);
}

/**
 * Returns the download url (link) for a given file.
 *
 * @param $fiel Relative path to the file
 * @return The complete url to the given file
 */
function ad_getDownloadURL($file)
{
	$siteUrl = get_option('siteurl');
	$relPath = get_option(OPTION_ROOT_DIR);
	return $siteUrl . "/" . $relPath . "/" . $file;
}

/**
 * Strips a trailing slash from a path string.
 * 
 * @param $path The path to strip the slash from
 * @return The path with stripped trailing slash
 */
function ad_stripTrailingSlash($path)
{
	if(substr($path, -1) == "/")
	{
		return substr($path, 0, -1);
	}

	return $path;
}


/* HELPER */

/**
 * Returns the absolute path to the donwload root dir (path indicated in
 * the Autodownload options page. Also formats the input field (option)
 * 'ad_rootDir' (makes it relative if absolute and strips the trailing slash).
 *
 * @return The absolute path to the download root directory
 */
function ad_getDownloadRootDir()
{
	$absPath = ABSPATH;
	$ad_rootDir = get_option(OPTION_ROOT_DIR);

	if(strlen($ad_rootDir) == 0)
	{
		update_option('ad_rootDir', "wp-content");	
	 	$absPath .= "wp-content";
	} else
	{	
		$rel_ad_rootDir = $ad_rootDir;

		// if path is not relative remove slash at pos = 0
		$pos = strpos($rel_ad_rootDir, "/"); 
		if(!is_bool($pos) && $pos == 0) // this was a bug in version 0.2
		{
			$rel_ad_rootDir = substr($rel_ad_rootDir, 1);
			update_option(OPTION_ROOT_DIR, $rel_ad_rootDir);	
		}

 		// also check that we have no trailing slash
		if(substr($rel_ad_rootDir, -1) == "/")
		{
			$rel_ad_rootDir = ad_stripTrailingSlash($rel_ad_rootDir, 1);
			update_option(OPTION_ROOT_DIR, $rel_ad_rootDir);
		}

		$absPath .= $rel_ad_rootDir;
	}

	return $absPath;
}

/**
 * Determines if the path indicated in the Autodownload options
 * is valid or not.
 * 
 * @return True is path exits
 */
function ad_isDownloadRootDirValid()
{
	$path = ad_getDownloadRootDir();
	return ad_isDirectory($path);
}

/**
 * Function used by usort.
 */
function ad_sortFilesBy_name($a, $b)
{
	if(array_key_exists('name', $a))
	{
	    return ($a['name'] < $b['name']) ? -1 : 1;
	}
        return 0;
}

/**
 * Function used by usort.
 */
function ad_sortFilesBy_date($a, $b)
{
	if(array_key_exists('date', $a))
	{
	    return ($a['date'] > $b['date']) ? -1 : 1;
	}

        return 0;
}

/**
 * Function used by array_walk. Trims the value of every array element.
 */
function ad_myTrim(&$value, $key)
{
	$value = trim($value);
}

?>
